@@ -3,6 +3,7 @@ import logging
 import math
 from datetime import datetime as timestamp
 from time import time as now
+from .metrics import Metrics
 
 
 def zero_heuristic():
@@ -11,13 +12,16 @@ def zero_heuristic():
 
 class DijkstraBestFirstSearch:
     def __init__(self, automated_planner):
+        self.time_start = now()
         self.automated_planner = automated_planner
+        self.metrics = Metrics()
         self.init = Node(
             self.automated_planner.initial_state,
             automated_planner,
             is_closed=False,
             is_open=True,
             heuristic=zero_heuristic,
+            metric=self.metrics,
         )
         self.open_nodes_n = 1
         self.nodes = dict()
@@ -28,33 +32,39 @@ class DijkstraBestFirstSearch:
         string = str(node.state)
         return string.split(sep, 1)[0] + ")"
 
-    def search(self):
-        opened_nodes = 0
+    def search(self, node_bound=float("inf")):
         self.automated_planner.logger.debug(
             "Search started at: " + str(timestamp.now())
         )
-        time_start = now()
         while self.open_nodes_n > 0:
             current_key = min(
                 [n for n in self.nodes if self.nodes[n].is_open],
                 key=(lambda k: self.nodes[k].f_cost),
             )
             current_node = self.nodes[current_key]
-
+            self.metrics.n_evaluated += 1
             if self.automated_planner.satisfies(
                 self.automated_planner.problem.goal, current_node.state
             ):
-                computation_time = now() - time_start
+                self.metrics.runtime = now() - self.time_start
                 self.automated_planner.logger.debug(
                     "Search finished at: " + str(timestamp.now())
                 )
-                return current_node, computation_time, opened_nodes
+                self.metrics.total_cost = current_node.g_cost
+                return current_node, self.metrics
 
             current_node.is_closed = True
             current_node.is_open = False
             self.open_nodes_n -= 1
 
+            if self.metrics.n_opened > node_bound:
+                break
+
             actions = self.automated_planner.available_actions(current_node.state)
+            if not actions:
+                self.metrics.deadend_states += 1
+            else:
+                self.metrics.n_expended += 1
             for act in actions:
                 child = Node(
                     state=self.automated_planner.transition(current_node.state, act),
@@ -64,8 +74,9 @@ class DijkstraBestFirstSearch:
                     heuristic=zero_heuristic,
                     is_closed=False,
                     is_open=True,
+                    metric=self.metrics,
                 )
-                opened_nodes += 1
+                self.metrics.n_generated += 1
                 child_hash = self.__hash(child)
                 if child_hash in self.nodes:
                     if self.nodes[child_hash].is_closed:
@@ -73,14 +84,18 @@ class DijkstraBestFirstSearch:
                     if not self.nodes[child_hash].is_open:
                         self.nodes[child_hash] = child
                         self.open_nodes_n += 1
+                        self.metrics.n_opened += 1
                     else:
                         if child.g_cost < self.nodes[child_hash].g_cost:
                             self.nodes[child_hash] = child
                             self.open_nodes_n += 1
+                            self.metrics.n_opened += 1
+                            self.metrics.n_reopened += 1
 
                 else:
                     self.nodes[child_hash] = child
+                    self.metrics.n_opened += 1
                     self.open_nodes_n += 1
-        computation_time = now() - time_start
+        self.metrics.runtime = now() - self.time_start
         self.automated_planner.logger.warning("!!! No path found !!!")
-        return None, computation_time, opened_nodes
+        return None, self.metrics
